#include "block_data.h"

const char *block_names[] =
{
    "Air",
    "Stone",
    "Dirt",
    "Sand",
    "Wood",
    "Leaves",
    "Grass"
};

//Only used if no texture support enabled
//I have to use float here because GLFix doesn't seem to work even with {}
float block_colors_raw[][3] =
{
    {0, 0, 0},
    {0.5, 0.5, 0.5},
    {0.3, 0.2, 0.2},
    {0.8, 0.7, 0.5},
    {0.4, 0.3, 0.2},
    {0.3, 0.4, 0},
    {0.2, 0.3, 0},
};

COLOR block_colors[sizeof(block_colors_raw)/sizeof(*block_colors_raw)];

//Maps location in texture atlas to block ID
BLOCK texture_atlas[][16] =
{
    { 0, BLOCK_STONE, BLOCK_DIRT, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, BLOCK_SAND, BLOCK_GRASS, BLOCK_WOOD, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, BLOCK_LEAVES, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
};

TextureAtlasEntry *block_textures = nullptr;

void init_blockData(TEXTURE *texture)
{
    if(texture != nullptr)
    {
        block_textures = new TextureAtlasEntry[sizeof(texture_atlas) / sizeof(BLOCK)];
        int fields_x = sizeof(*texture_atlas)/sizeof(**texture_atlas);
        int fields_y = sizeof(texture_atlas)/sizeof(*texture_atlas);
        int field_width = texture->width / fields_x;
        int field_height = texture->height / fields_y;

        for(int y = 0; y < fields_y; y++)
            for(int x = 0; x < fields_x; x++)
                block_textures[texture_atlas[y][x]] = textureArea(x * field_width, y * field_height, field_width, field_height, texture);
    }

    for(unsigned int i = 0; i < sizeof(block_colors_raw)/sizeof(*block_colors_raw); i++)
        block_colors[i] = colorRGB(block_colors_raw[i][0], block_colors_raw[i][1], block_colors_raw[i][2]);
}

void uninit_blockData()
{
    delete[] block_textures;
}
