#ifndef CHUNK_H
#define CHUNK_H

#include <cstdint>
#include <vector>

#include "world.h"
#include "gl.h"
#include "block_data.h"
#include "aabb.h"

struct Position {
    GLFix x, y, z;
};

struct IndexedVertex {
    int pos;
    GLFix u, v;
    COLOR c;
};

class Chunk
{
public:
    Chunk(int x, int y, int z, World *parent);
    void render();
    void setDirty() { render_dirty = true; }
    BLOCK getLocalBlock(int x, int y, int z);
    AABB &getAABB() { return aabb; }
    bool intersects(AABB &other);

    static constexpr int SIZE = 16;

    const int x, y, z;

private:
    void makeTree(unsigned int x, unsigned int y, unsigned int z);

    void setLight(int x, int y, int z, uint8_t value)
    {
        light[x + 1][y + 1][z + 1] = value;
    }

    uint8_t getLight(int x, int y, int z)
    {
        return light[x + 1][y + 1][z + 1];
    }

    void generateLight(int x, int y, int z, uint8_t value);
    inline const GLFix lightToFix(const uint8_t l)
    {
        return GLFix(l) / GLFix(30);
    }

    BLOCK getGlobalBlockRelative(int x, int y, int z);

    int getPosition(GLFix x, GLFix y, GLFix z);
    void addVertex(const GLFix x, const GLFix y, const GLFix z, GLFix u, GLFix v, const COLOR c);
    void buildGeometry();

    const GLFix abs_x, abs_y, abs_z;
    World * const parent;
    AABB aabb;
    uint8_t light[SIZE + 2][SIZE + 2][SIZE + 2];
    BLOCK blocks[SIZE][SIZE][SIZE];

    bool render_dirty = true;
    int pos_indices[SIZE + 1][SIZE + 1][SIZE + 1];
    std::vector<Position> positions;
    std::vector<Position> positions_transformed;
    std::vector<IndexedVertex> vertices;
};

#endif // CHUNK_H
