#include "gl.h"
#include "fastmath.h"
#include "chunk.h"
#include "glbox.h"

#ifdef TEXTURE_SUPPORT
    #include "terrain.h"
#endif

static GLFix x = 500, y = 2000, z = 500;
static GLFix old_x = 0, old_y = 0, old_z = 0;
static const GLFix player_width = 80, player_height = 180, eye_pos = 150, incr = 20;
static GLFix xr = 0, yr = 0;

static void getForward(GLFix *x, GLFix *z)
{
    *x = fast_sin(yr) * incr;
    *z = fast_cos(yr) * incr;
}

static void getRight(GLFix *x, GLFix *z)
{
    *x = fast_sin((yr + 90).normalise_angle()) * incr;
    *z = fast_cos((yr + 90).normalise_angle()) * incr;
}

int main()
{
    if(!lcd_isincolor())
    {
        show_msgbox("Error", "Your LCD hasn't got enough colors :-P");
        return EXIT_FAILURE;
    }

    nglInit();

    COLOR* const screen_buffer = new COLOR[WIDTH*HEIGHT];
    nglSetBuffer(screen_buffer);

    #ifdef TEXTURE_SUPPORT
        init_blockData(&terrain);
        glBindTexture(&terrain);
    #else
        init_blockData(nullptr);
    #endif

    glLoadIdentity();

    bool finished = false;

    World world;

    while(!finished)
    {
        glColor3f(0.1f, 0.1f, 1); //Blue background
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

        glPushMatrix();

        nglRotateX(GLFix(359) - xr.normalise_angle());
        nglRotateY(GLFix(359) - yr.normalise_angle());
        glTranslatef(-x, -y - eye_pos, -z);

        world.setPosition(x, y, z);

        world.render();

        glPopMatrix();

        nglDisplay();

        AABB aabb(x - 40, y, z - 40, x + 40, y + 200, z + 40);

        if(isKeyPressed(KEY_NSPIRE_ESC))
            finished = true;

        if(isKeyPressed(KEY_NSPIRE_8))
        {
            GLFix dx1, dz1;
            getForward(&dx1, &dz1);

            x += dx1;
            z += dz1;
        }
        if(isKeyPressed(KEY_NSPIRE_2))
        {
            GLFix dx1, dz1;
            getForward(&dx1, &dz1);

            x -= dx1;
            z -= dz1;
        }
        if(isKeyPressed(KEY_NSPIRE_4))
        {
            GLFix dx1, dz1;
            getRight(&dx1, &dz1);

            x -= dx1;
            z -= dz1;
        }
        if(isKeyPressed(KEY_NSPIRE_6))
        {
            GLFix dx1, dz1;
            getRight(&dx1, &dz1);

            x += dx1;
            z += dz1;
        }
        if(isKeyPressed(KEY_NSPIRE_SPACE))
            y += incr;
        if(isKeyPressed(KEY_NSPIRE_SHIFT))
            y -= incr;

        if(isKeyPressed(KEY_NSPIRE_LEFT))
            yr -= incr/3;
        if(isKeyPressed(KEY_NSPIRE_RIGHT))
            yr += incr/3;
        if(isKeyPressed(KEY_NSPIRE_UP))
            xr -= incr/3;
        if(isKeyPressed(KEY_NSPIRE_DOWN))
            xr += incr/3;
    }

    nglUninit();

    delete[] screen_buffer;

    return EXIT_SUCCESS;
}
