#include "chunk.h"

#include "world.h"

World::World()
{
    setPosition(0, 0, 0);
}

World::~World()
{
    for(Chunk *c : map)
        delete c;
}

BLOCK World::getBlock(int x, int y, int z)
{
    if(y < 0)
        return BLOCK_STONE;

    int chunk_x = floor(float(x) / Chunk::SIZE), chunk_y = floor(float(y) / Chunk::SIZE), chunk_z = floor(float(z) / Chunk::SIZE);

    Chunk *c = findChunk(chunk_x, chunk_y, chunk_z);
    if(!c)
        return BLOCK_AIR;

    return c->getLocalBlock(x - chunk_x*Chunk::SIZE, y - chunk_y * Chunk::SIZE, z - chunk_z * Chunk::SIZE);
}

void World::setPosition(int x, int y, int z)
{
    (void) y;
    int chunk_x = floor(float(x) / (Chunk::SIZE * BLOCK_SIZE)), chunk_z = floor(float(z) / (Chunk::SIZE * BLOCK_SIZE));

    if(chunk_x != cen_x || chunk_z != cen_z)
    {
        visible_chunks.clear();

        for(int x = chunk_x - field_of_view; x <= chunk_x + field_of_view; x++)
                for(int z = chunk_z - field_of_view; z <= chunk_z + field_of_view; z++)
                {
                    Chunk *c = findChunk(x, 0, z);
                    if(!c)
                        c = generateChunk(x, 0, z);

                    visible_chunks.push_back(c);
                }

        cen_x = chunk_x;
        cen_z = chunk_z;
    }
}

void World::render()
{
    for(Chunk *c : visible_chunks)
        c->render();
}

Chunk* World::findChunk(int x, int y, int z)
{
    for(Chunk *c : map)
        if(c->x == x && c->y == y && c->z == z)
            return c;

    return nullptr;
}

Chunk* World::generateChunk(int x, int y, int z)
{
    if(Chunk *c = findChunk(x - 1, y, z))
        c->setDirty();
    if(Chunk *c = findChunk(x + 1, y, z))
        c->setDirty();
    if(Chunk *c = findChunk(x, y - 1, z))
        c->setDirty();
    if(Chunk *c = findChunk(x, y + 1, z))
        c->setDirty();
    if(Chunk *c = findChunk(x, y, z - 1))
        c->setDirty();
    if(Chunk *c = findChunk(x, y, z + 1))
        c->setDirty();

    Chunk *c = new Chunk(x, y, z, this);
    map.push_back(c);

    return c;
}
